package com.link.core.util.entry;

import com.alibaba.fastjson.JSON;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class EntityBuilder<T> {
   private final Supplier<T> constructor;
   private final List<Consumer<T>> dInjects = new ArrayList();

   private EntityBuilder(Supplier<T> constructor) {
      this.constructor = constructor;
   }

   public static <T> EntityBuilder<T> builder(Supplier<T> constructor) {
      return new EntityBuilder(constructor);
   }

   public <P1> EntityBuilder<T> with(DInjectConsumer<T, P1> consumer, P1 p1) {
      Consumer<T> c = (instance) -> {
         consumer.accept(instance, p1);
      };
      this.dInjects.add(c);
      return this;
   }

   public T build() {
      T instance = this.constructor.get();
      this.dInjects.forEach((s) -> {
         s.accept(instance);
      });
      return instance;
   }

   public static void main(String[] args) {
      Student student = (Student)builder(Student::new).with(Student::setName, "张三").with(Student::setAge, 20).build();
      System.out.println(JSON.toJSONString(student));
   }

   static class Student {
      private String name;
      private Integer age;

      public String getName() {
         return this.name;
      }

      public void setName(String name) {
         this.name = name;
      }

      public Integer getAge() {
         return this.age;
      }

      public void setAge(Integer age) {
         this.age = age;
      }
   }

   @FunctionalInterface
   public interface DInjectConsumer<T, P1> {
      void accept(T t, P1 p1);
   }
}
